function [SER,RPopts]=FRPS_complex(SER,s,s2,s3,RPopts)

% =========================================================================
% This code modifies the function FRPS.m of the Matrix Fitting Toolbox 
% available at
%    https://www.sintef.no/projectweb/vectfit/ 
%
% In particular, the function FRPS_complex.m has the same input/output 
% format of the function FRPS.m, which is described in the following 
% comments.
%
% NOTE: The use of this program is limited to NON-COMMERCIAL usage only.
% If the program code (or a modified version) is used in a scientific work,
% then reference should be made to the following:
%    
% [1] D. Spina, Y. Ye, D. Deschrijver, W. Bogaerts and T. Dhaene, Complex  
%  Vector Fitting Toolbox: a software package for the modeling and 
%  simulation of general linear and passive baseband systems, submitted to 
%  Electronics Letters, 2020
% 
% [2] B. Gustavsen, Fast passivity enforcement for S-parameter models by 
% perturbation of residue matrix eigenvalues, IEEE Trans. Adv. Packag. 33, 
% 257265 (2010).
%
%
% ========================================================================

% %%%%%%%%%%%%%%%%%%%%%%%%% Comments function FRPS.m of the Matrix Fitting Toolbox %%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %
% % [SER,RPopts]=FRPS(SER,s,s2,s3,RPopts);
% %
% % This file is part of the Matrix Fitting Toolbox, v1.
% % Filename: FRPS.m
% % Package: Matrix_Fitting_Toolbox_1.zip.
% % Programmed by B. Gustavsen. October 08, 2008.
% %
% %      ======================================================
% %      =   Routine: FRPS.m                                  =
% %      =   Version 1.0                                      =
% %      =   Last revised: 08.10.2008                         =
% %      =   Bjorn Gustavsen                                  =
% %      =   SINTEF Energy Research, N-7465 Trondheim, NORWAY =
% %      =   bjorn.gustavsen@sintef.no                        =
% %      ======================================================
% %
% %  PURPOSE : Modify elements in SER of rational approximation  to enforce passivity
% %            of S-parameter model at frequency samples in s2 and s3, such that the
% %            perturbation of the model is minimized at samples in s.
% %
% %  Reference:
% %     B. Gustavsen and A. Semlyen, "Fast passivity assessment for S-parameter rational
% %     models via a half-size test matrix",
% %     IEEE Trans. Microwave Theory and Techniques, accepted for publication.

auxflag=RPopts.auxflag;
weightfactor=RPopts.weightfactor;
weightparam=RPopts.weightparam;
TOL=RPopts.TOLGD;


SERA=SER.poles;
SERAA=SER.A;
SERBB=SER.B;
SERCC=SER.C;
[m,n]=size(SERA);
if m<n
    SERA=SERA.'; %M ha kolonne-vektor
end
SERC=SER.R;
SERD=SER.D;
SERE=SER.E;


if ~isfield(RPopts,'H')
    RPopts.H=[];
    RPopts.oldDflag=-1;
end

%d=eig(SERD);eigD=d;
[U,S,V]=svd(SERD,0);
if any(diag(S)>1)
    Dflag=1; %Will perturb D-matrix
    [VD,eigD]=eig(SERD); invVD=VD^(-1); eigD=diag(eigD);
else
    Dflag=0;
end

SERCnew=SERC; SERDnew=SERD; SEREnew=SERE;

N=length(SERA);

bigB=[];
bigc=[];
Ns=length(s);
Ns2=length(s2);
Nc=length(SERD);
%!!Nc2=Nc*Nc;
Nc2=Nc*(Nc+1)/2;
I=eye(Nc);
M2mat=[];


%=======================================================
% Finding out which poles are complex :
%=======================================================
% LS problem:
cindex=zeros(1,N);
for m=1:N
    if imag(SERA(m))~=0
        cindex(m)=1;
    end
end
Nr=N+length(find(cindex==1));%%%%% the total number of residues for each element in S, where the real and imaginary parts of one residue are counted separately;




if isempty(RPopts.H) 
    
    if RPopts.outputlevel==1, disp('    Building system equation (once)...'), end
    %========================================
    % LOOP FOR LEAST SQUARES PROBLEM:
    %========================================
    if (Dflag)==1
        H=zeros(Nc*(Nr+1));
        Mmat=zeros(Nc2,Nc*(Nr+1));
    else
        H=zeros(Nc*(Nr));
        Mmat=zeros(Nc2,Nc*(Nr));
    end
    count_r=1;  
    for m=1:N
        R=SERC(:,:,m);
        if cindex(m)==0      %real pole
            [V,D]=eig(R);
            bigV(1:Nc,(count_r-1)*Nc+1:count_r*Nc)=V;
            biginvV(1:Nc,(count_r-1)*Nc+1:count_r*Nc)=V^(-1);
            bigD(:,count_r)=diag(D);
            count_r=count_r+1;
        else                 %complex pole
            R_real=real(R); 
            [V,D]=eig(R_real);
            bigV(1:Nc,(count_r-1)*Nc+1:count_r*Nc)=V;
            biginvV(1:Nc,(count_r-1)*Nc+1:count_r*Nc)=V^(-1);
            bigD(:,count_r)=diag(D);
            count_r=count_r+1;
            R_imag=imag(R);
            [V,D]=eig(R_imag);
            bigV(1:Nc,(count_r-1)*Nc+1:count_r*Nc)=V;
            biginvV(1:Nc,(count_r-1)*Nc+1:count_r*Nc)=V^(-1);
            bigD(:,count_r)=diag(D);
            count_r=count_r+1;
        end
    end
    
    %Beregning av kolonne skalering:
    if Dflag==1,Escale=zeros(Nr+1,1);else,Escale=zeros(Nr,1);end
    count_r=1;
    for m=1:N
        if cindex(m)==0      %real pole
            Escale(count_r)=1/norm( 1./(s-SERA(m)) );
            count_r=count_r+1;
        elseif cindex(m)==1  %complex pole
            Escale(count_r)=1/norm(1./(s-SERA(m)));
        	count_r=count_r+1;
            Escale(count_r)=1/norm(1i./(s-SERA(m)));
            count_r=count_r+1;
        end
    end
    if Dflag==1,Escale(Nr+1)=1./sqrt(Ns); end
    
    %Escale=1+0.*Escale;
    if Dflag==1, dum=Nr+1;else,dum=Nr;end
    Escale2=zeros(Nc*dum,1);
    for m=1:dum
        Escale2((m-1)*Nc+1:Nc*dum)=Escale(m);
    end
    RPopts.Escale=Escale;
    RPopts.Escale2=Escale2;
    %Creating indices to lower triangle:
    indices=zeros(Nc2,1);
    tell=0;
    for col=1:Nc
        for row=col:Nc
            tell=tell+1;
            indices(tell)=(col-1)*Nc+row;
        end
    end
    for k=1:Ns
        %k
        sk=s(k);
        % Calculating matrix Mmat (coefficients for LS-problem)
        offs=0;
        %Yfit=Yfitcalc(sk,SERA,SERC,SERD,SERE);
        Yfit=fitcalcABCDE(sk,diag(SERAA),SERBB,SERCC,SERD,SERE);
        %y=SERD(row,col)+sk*SERE(row,col);
        %y=y+sum(squeeze(SERC(row,col,1:N))./(s(k)-SERA(1:N)));
        if weightparam==1
            weight=ones(Nc,Nc);
        elseif weightparam==2
            weight=1./abs(Yfit);
        elseif weightparam==3
            weight=1./sqrt(abs(Yfit));
        elseif weightparam==4
            weight=ones(Nc,Nc)./norm(abs(Yfit));
        elseif weightparam==5
            weight=ones(Nc,Nc)./sqrt(norm(abs(Yfit)));
        else
            'ERROR!!!!!!!!!'
            weight=1;
        end
        
        
        count_r=1;
        for m=1:N

            if cindex(m)==0 %real pole
                V=(bigV(:,(count_r-1)*Nc+1:count_r*Nc));
                invV=V^(-1);%squeeze(biginvV(:,(m-1)*Nc+1:m*Nc));
                dum=Escale(count_r)/(sk-SERA(m));
                count_r=count_r+1;
                for egenverdi=1:Nc
                    gamm=V(:,egenverdi)*invV(egenverdi,:);
                    gammx=gamm.*weight.*dum;
                    Mmat(:,offs+egenverdi)=gammx(indices);
                end 
                offs=offs + (Nc);
            elseif cindex(m)==1  %complex pole
                V=(bigV(:,(count_r-1)*Nc+1:count_r*Nc));
                invV=V^(-1);%squeeze(biginvV(:,(m-1)*Nc+1:m*Nc));
                dum=Escale(count_r)*(1/(sk-SERA(m)));
                count_r=count_r+1;
                for egenverdi=1:Nc
                    gamm=V(:,egenverdi)*invV(egenverdi,:);
                    gammx=gamm.*weight.*dum;
                    Mmat(:,offs+egenverdi)=gammx(indices);
                end 
                offs=offs + (Nc);
                V=(bigV(:,(count_r-1)*Nc+1:count_r*Nc));
                invV=V^(-1);%squeeze(biginvV(:,(m-1)*Nc+1:m*Nc));
                dum=Escale(count_r)*(1i/(sk-SERA(m)));
                count_r=count_r+1;
                for egenverdi=1:Nc
                    gamm=V(:,egenverdi)*invV(egenverdi,:);
                    gammx=gamm.*weight.*dum;
                    Mmat(:,offs+egenverdi)=gammx(indices);
                end
                offs=offs + (Nc);
            end
        end 
        
        
        if Dflag==1
            for egenverdi=1:Nc %Egenverdi til residue-matrise
                gamm=Escale(Nr+1)*VD(:,egenverdi)*invVD(egenverdi,:); %Ytre-produkt
                tell=0;
                for row=1:Nc
                    for col=row:Nc
                        tell=tell+1;
                        if row==col
                            faktor=weight(row,col);
                        else
                            faktor=2*weight(row,col);
                        end
                        Mmat(tell,offs+egenverdi)=gamm(row,col)*faktor;
                    end
                end
            end
        end
        
        XX=[real(Mmat);imag(Mmat)];
        H=H+XX.'*XX;
        
    end 
    
    %========================================
    % INTRODUCING SAMPLES OUTSIDE LS REGION: ONE SAMPLE PER POLE (s4)
    %========================================
    if auxflag==1
        s4=[];
        tell=0;
        for m=1:length(SERA)
                if imag(SERA(m))>s(Ns)/1j
                    tell=tell+1;
                    s4(tell)=1j*imag(SERA(m));
                elseif imag(SERA(m))<s(1)/1j
                    tell=tell+1;
                    s4(tell)=1j*imag(SERA(m));
                end
        end
        Ns4=length(s4);
        
        
        for k=1:1:Ns4
            sk=s4(k);
            % Calculating matrix Mmat (coefficients for LS-problem)
            tell=0;
            offs=0;
            Yfit=fitcalcPRE(sk,SERA,SERC,SERD,SERE);
            %y=SERD(row,col)+sk*SERE(row,col);
            %y=y+sum(squeeze(SERC(row,col,1:N))./(s(k)-SERA(1:N)));
            if weightparam==1
                weight=ones(Nc,Nc);
            elseif weightparam==2
                weight=1./abs(Yfit);
            elseif weightparam==3
                weight=1./sqrt(abs(Yfit));
            elseif weightparam==4
                weight=ones(Nc,Nc)./norm(abs(Yfit));
            elseif weightparam==5
                weight=ones(Nc,Nc)./sqrt(norm(abs(Yfit)));
            else
                'ERROR!!!!!!!!!'
                weight=1;
            end
            weight=weight*weightfactor;
            
            count_r=1;
            for m=1:N

                if cindex(m)==0 %real pole
                    V=squeeze(bigV(:,(m-1)*Nc+1:m*Nc));
                    invV=V^(-1);%squeeze(biginvV(:,(m-1)*Nc+1:m*Nc));
                    dum=Escale(count_r)/(sk-SERA(m));
                    count_r=count_r+1;
                    for egenverdi=1:Nc
                        tell=0;
                        gamm=V(:,egenverdi)*invV(egenverdi,:);
                        for row=1:Nc
                            for col=row:Nc
                                tell=tell+1;
                                if row==col
                                    faktor=weight(row,col);
                                else
                                    faktor=2*weight(row,col);
                                end
                                Mmat(tell,offs+egenverdi)=gamm(row,col)*faktor*dum;
                            end
                        end
                    end 
                    offs=offs + (Nc);
                elseif cindex(m)==1  %complex pole
                    V=squeeze(bigV(:,(m-1)*Nc+1:m*Nc));
                    invV=V^(-1);%squeeze(biginvV(:,(m-1)*Nc+1:m*Nc));
                    dum=Escale(count_r)*(1/(sk-SERA(m)));
                    count_r=count_r+1;
                    for egenverdi=1:Nc
                        tell=0;
                        gamm=V(:,egenverdi)*invV(egenverdi,:);
                        for row=1:Nc
                            for col=row:Nc
                                tell=tell+1;
                                if row==col
                                    faktor=weight(row,col);
                                else
                                    faktor=2*weight(row,col);
                                end
                                Mmat(tell,offs+egenverdi)=gamm(row,col)*faktor*dum;
                            end
                        end
                    end %for egenverdi=1:Nc
                    offs=offs + (Nc);
                    V=squeeze(bigV(:,(m-1)*Nc+1:m*Nc));
                    invV=V^(-1);%squeeze(biginvV(:,(m-1)*Nc+1:m*Nc));
                    dum=Escale(count_r)*(1i/(sk-SERA(m)));
                    count_r=count_r+1;
                    for egenverdi=1:Nc
                        tell=0;
                        gamm=V(:,egenverdi)*invV(egenverdi,:);
                        for row=1:Nc
                            for col=row:Nc
                                tell=tell+1;
                                if row==col
                                    faktor=weight(row,col);
                                else
                                    faktor=2*weight(row,col);
                                end
                                Mmat(tell,offs+egenverdi)=gamm(row,col)*faktor*dum;
                            end
                        end
                    end %for egenverdi=1:Nc
                    offs=offs + (Nc);
                end
            end %for m=1:N
            
            if Dflag==1
                for egenverdi=1:Nc %Egenverdi til residue-matrise
                    gamm=Escale(Nr+1)*VD(:,egenverdi)*invVD(egenverdi,:); %Ytre-produkt
                    tell=0;
                    for row=1:Nc
                        for col=row:Nc
                            tell=tell+1;
                            if row==col
                                faktor=weight(row,col);
                            else
                                faktor=2*weight(row,col);
                            end
                            Mmat(tell,offs+egenverdi)=gamm(row,col)*faktor;
                        end
                    end
                end
            end

            H=H+ real(Mmat.')*real(Mmat) + imag(Mmat.')*imag(Mmat);
        end 
        
        
    end 
    
    
    RPopts.H=H; clear H
    RPopts.bigV=bigV;
    RPopts.biginvV=biginvV;
    if RPopts.outputlevel==1, disp('    Done'), end
    
else 
    bigV=RPopts.bigV;
    biginvV=RPopts.biginvV;
    if Dflag~=RPopts.oldDflag
        RPopts.H=RPopts.H(1:Nc*(N+Dflag),1:Nc*(N+Dflag));
        RPopts.Escale=RPopts.Escale((N+Dflag));
    end
    
end 

Escale=RPopts.Escale;
Escale2=RPopts.Escale2;

Mmat2=zeros(Nc,Nc*(Nr+Dflag));

%========================================
% LOOP FOR CONSTRAINT PROBLEM, TYPE #1 (violating eigenvalues in s2):
%========================================

viol_S=[];
viol_D=[];

for k=1:Ns2
    sk=s2(k);
    
    %   for row=1:Nc
    %     for col=1:Nc
    %       Y(row,col)=SERD(row,col)+sk*SERE(row,col);
    %       Y(row,col)=Y(row,col)+sum(squeeze(SERC(row,col,1:N))./(sk-SERA(1:N)));
    %     end
    %   end
    Y=fitcalcABCDE(sk,diag(SERAA),SERBB,SERCC,SERD,SERE);
    
    [U,S,V]=svd(Y,0);
    SS=diag(S);
    Qa=0.5*V^(-1);
    Qb=0.5*U^(-1);
    Ta=V;
    Tb=U;
    
    if max(diag(S))>1  %any violations
        % Calculating matrix M2mat; matrix of partial derivatives :
        tell=0;
        offs=0;
        count_r=1;
        for m=1:N
            
            if cindex(m)==0 %real pole
                VV=bigV(:,(count_r-1)*Nc+1:count_r*Nc);
                count_r=count_r+1;
                for egenverdi=1:Nc
                    p=VV(:,egenverdi);
                    for row=1:Nc %singular values
                        k1=(Qb(row,:)*p)*(p.'*Ta(:,row));
                        Mmat2(row,offs+egenverdi)=2*k1/(sk-SERA(m));
                    end
                end
                offs=offs + (Nc);
            elseif cindex(m)==1  %complex pole, 1st part
                VV1=      bigV(:,(count_r-1)*Nc+1:count_r*Nc);
                VV2=      bigV(:,(count_r+1-1)*Nc+1:(count_r+1)*Nc);
                count_r=count_r+2;
                f=1/(sk-SERA(m));
                g=1i/(sk-SERA(m));
                for egenverdi=1:Nc
                    p1=VV1(:,egenverdi);
                    p2=VV2(:,egenverdi);
                    for row=1:Nc %singular values
                        k1=2*f*(Qb(row,:)*p1)*(p1.'*Ta(:,row));
                        k2=2*g*(Qb(row,:)*p2)*(p2.'*Ta(:,row));
                        Mmat2(row,offs+egenverdi)=k1;
                        Mmat2(row,offs+egenverdi+Nc)=k2;
                    end
                end 
                offs=offs + 2*(Nc);
            end
        end 
        
        if Dflag==1
            for egenverdi=1:Nc
                gamm=VD(:,egenverdi)*invVD(egenverdi,:);
                for row=1:Nc %singular values
                    k1= Qb(row,:)*gamm*Ta(:,row);
                    Mmat2(row,offs+egenverdi)=2*k1;
                end
            end 
        end
        
        B=Mmat2;
        dels=SS;
        for n=1:Nc   %ustabilitet?
            if dels(n)>1
                bigB=[bigB;B(n,:)];
                bigc=[bigc;-TOL+1-dels(n)];
                viol_S=[viol_S;dels(n)];
            end
        end
        
    end 
    
end 

%bigB=real(bigB);
 
%========================================
% LOOP FOR CONSTRAINT PROBLEM, TYPE #2: (all eigenvalues in s3):
%========================================
Ns3=length(s3);
for k=1:Ns3
    sk=s3(k);
    
    Y=fitcalcABCDE(sk,diag(SERAA),SERBB,SERCC,SERD,SERE);
    
    [U,S,V]=svd(Y,0);
    SS=diag(S);
    Qa=0.5*V^(-1);
    Qb=0.5*U^(-1);
    Ta=V;
    Tb=U;
    
    if max(diag(S))>1  %any violations
        % Calculating matrix M2mat; matrix of partial derivatives :
        offs=0;
        count_r=1;
        for m=1:N
            if cindex(m)==0 %real pole
                VV=bigV(:,(count_r-1)*Nc+1:count_r*Nc);
                count_r=count_r+1;
                for egenverdi=1:Nc
                    p=VV(:,egenverdi);
                    for row=1:Nc %singular values
                        k1=(Qb(row,:)*p)*(p.'*Ta(:,row));
                        Mmat2(row,offs+egenverdi)=2*k1/(sk-SERA(m));
                    end
                end
                offs=offs + (Nc);
            elseif cindex(m)==1  %complex pole, 1st part
                VV1=      bigV(:,(count_r-1)*Nc+1:count_r*Nc);
                VV2=      bigV(:,(count_r+1-1)*Nc+1:(count_r+1)*Nc);
                count_r=count_r+2;
                f=(1/(sk-SERA(m)));
                g=(1i/(sk-SERA(m)));
                for egenverdi=1:Nc
                    p1=VV1(:,egenverdi);
                    p2=VV2(:,egenverdi);
                    for row=1:Nc %singular values
                        k1=2*f*(Qb(row,:)*p1)*(p1.'*Ta(:,row));
                        k2=2*g*(Qb(row,:)*p2)*(p2.'*Ta(:,row));
                        Mmat2(row,offs+egenverdi)=k1;
                        Mmat2(row,offs+egenverdi+Nc)=k2;
                    end
                end 
                offs=offs + 2*(Nc);
            end
        end 
        
        if Dflag==1
            for egenverdi=1:Nc
                gamm=Escale(Nr+1)*VD(:,egenverdi)*invVD(egenverdi,:); %%%%%%% Yinghao 
                for row=1:Nc %singular values
                    k1= Qb(row,:)*gamm*Ta(:,row);
                    Mmat2(row,offs+egenverdi)=2*k1;
                end
            end 
        end
        B=Mmat2;
        dels=SS;
        for n=1:Nc   %ustabilitet?
            %if dels(n)>1
            bigB=[bigB;B(n,:)];
            bigc=[bigc;-TOL+1-dels(n)];
            viol_S=[viol_S;dels(n)];
            %end
        end
        
    end 
    
end 

%Legger til constraint for evt. sing.-verdier i D >1
if Dflag==1
    %!![U,S,V]=svd(Escale(m+1)*SER.D,0);
    [U,S,V]=svd(SER.D,0);
    SS=diag(S);
    Qa=0.5*V^(-1);
    Qb=0.5*U^(-1);
    Ta=V;
    Tb=U;
    dumB=zeros(sum(SS>1),Nc*Nr+Nc);
    dumc=zeros(sum(SS>1),1);
    for egenverdi=1:Nc
        tell=0;
        gamm=VD(:,egenverdi)*invVD(egenverdi,:);
        for row=1:Nc %singular values
            if SS(row)>1
                koko=[Qa(row,:) Qb(row,:)]*[Ta(:,row);Tb(:,row)];
                k1= Qb(row,:)*gamm*Ta(:,row);
                k2= Qa(row,:)*gamm*Tb(:,row);
                tell=tell+1;
                dumB(tell,Nc*Nr+egenverdi)=(1/koko)*( k1 + k2 );
                dumc(tell)=-TOL+1-SS(row);
                viol_S=[viol_S; SS(row)];
                viol_D=[viol_D; SS(row)];
            end
            %end
        end
    end 
    bigB=[bigB;dumB];
    bigc=[bigc;dumc];
end


if length(bigB)==0
    return   %No passivity violations
end


c=bigc;


bigB=real(bigB);
for col=1:length(RPopts.H)
    if length(bigB)>0
        bigB(:,col)=bigB(:,col).*RPopts.Escale2(col);
    end
end

%bigB=real(bigB)


ff=zeros(length(RPopts.H),1);

clear bigA;
%disp('Starting quadprog...')

warning off


if RPopts.solver=='QUADPROG'
    %P_options = optimoptions('quadprog','Display','off');
    %[dx,lambda]=quadprog(RPopts.H,ff,bigB,bigc,[],[],[],[],[],P_options); %quadprog(H,f,A,b,Aeq,beq,lb,ub,x0,options)
    [dx,lambda]=quadprog(RPopts.H,ff,bigB,bigc);
elseif RPopts.solver=='CPLEX   '
    
    c=0*ff;
    H0=c.';
    
    %F=sparse(H);
    A=-bigB;
    
    x_0=zeros(length(H(:,1)));
    b_U=bigc;
    b_L = -inf*ones(length(b_U),1);
    x_L = -inf*ones(length(x_0),1);
    x_U = inf*ones(length(x_0),1);
    
    Prob = qpAssign(F,c,A,b_L,b_U,x_L,x_U,x_0,'dust');
    Prob.MIP.cpxControl.QPMETHOD = 4;
    Prob.MIP.cpxControl.BARALG = 2;
    PriLev = 0;
    Prob.PriLevOpt = 0;
    [x, slack, v, rc, f_k] = cplex(c, A, x_L, x_U, b_L, b_U, ...
        Prob.MIP.cpxControl, [], PriLev, Prob, [], [], [], [], ...
        [], [], F);
    dx=x;
    
end
%disp(['Elapsed time: ' num2str(toc) ' sec'])
warning on

dx=dx.*RPopts.Escale2(1:Nc*(Nr+Dflag));


%Oppdaterer egenverdiene og SERC:
count_r=1;
for m=1:N
    if cindex(m)==0 %real pole
        D1=diag(dx((count_r-1)*Nc+1:count_r*Nc));
        SERCnew(:,:,m)=SERCnew(:,:,m) +( bigV(:,(count_r-1)*Nc+1:count_r*Nc)*D1*biginvV(:,(count_r-1)*Nc+1:count_r*Nc) );
        count_r=count_r+1;
    elseif cindex(m)==1  %complex pole
        GAMM1=bigV(:,(count_r-1)*Nc+1:count_r*Nc);
        GAMM2=bigV(:,(count_r+1-1)*Nc+1:(count_r+1)*Nc);
        invGAMM1=biginvV(:,(count_r-1)*Nc+1:count_r*Nc);
        invGAMM2=biginvV(:,(count_r+1-1)*Nc+1:(count_r+1)*Nc);
        
        D1=diag(dx((count_r-1)*Nc+1:count_r*Nc));
        D2=diag(dx((count_r+1-1)*Nc+1:(count_r+1)*Nc));
        R1=real(SERC(:,:,m));
        R2=imag(SERC(:,:,m));
        R1new=R1 +GAMM1*D1*invGAMM1;
        R2new=R2 +GAMM2*D2*invGAMM2;
        SERCnew(:,:,m)  =R1new+1i*R2new;
        count_r=count_r+2;
    end
end

if Dflag==1
    DD=diag(dx(Nr*Nc+1:(Nr+1)*Nc));
    SERDnew=SERDnew+VD*DD*invVD;
end

%==========================
SERDnew=(SERDnew+SERDnew.')/2;

for m=1:N
    SERCnew(:,:,m)=(SERCnew(:,:,m)+SERCnew(:,:,m).')/2;
end
%==========================
SER.R=SERCnew;
SER.D=SERDnew;
SER.E=zeros(Nc);
[SER]=pr2ss(SER);

RPopts.oldDflag=Dflag;




end 


